using System;
using System.Security.Cryptography;
using Autofac;
using FitConnect;
using FitConnect.Encryption;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using MockContainer;
using Newtonsoft.Json;
using NUnit.Framework;

namespace SenderTest;

public class JweTest {
    private IContainer _container;
    private ILogger<JweTest> _logger;
    private Sender _sender;
    private MockSettings _settings;


    [SetUp]
    public void SetUp() {
        _logger = LoggerFactory.Create(cfg => cfg.AddConsole())
            .CreateLogger<JweTest>();

        _sender = Sender.Create(
            FitConnectEnvironment.Development,
            _logger);

        _container = Container.Create();
        _settings = _container.Resolve<MockSettings>();
    }


    [Test]
    public void TestJwe() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var dummyText = JsonConvert.SerializeObject(new { Title = "Value", Content = "content" });
        Console.WriteLine(dummyText);
        var cypher = engine.Encrypt(dummyText);
        Console.WriteLine(cypher);
        var plain = engine.Decrypt(cypher);
        Console.WriteLine(plain.plainText);

        // Assert
        plain.plainText.Should().Be(dummyText);
    }

    [Test]
    public void TestJwe_withBytes() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var plainBytes = RandomNumberGenerator.GetBytes(8192);
        var cypher = engine.Encrypt(plainBytes);
        var plain = engine.Decrypt(cypher);


        // Assert
        plain.plainBytes.Should().BeEquivalentTo(plainBytes);
    }

    [Test]
    public void TestJwe_withBytes2() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var plainBytes = RandomNumberGenerator.GetBytes(512);
        var cypher = engine.Encrypt(Convert.ToBase64String(plainBytes));
        var plain = engine.Decrypt(cypher);

        // Assert
        plain.plainText.Should().BeEquivalentTo(Convert.ToBase64String(plainBytes));
    }

    [TestCase("eyJ6aXAiOiJERUYiLCJraWQiOiJEOUdrck51S2dhRWJSOHM0VkotdEtqRVBYalJpQ0FOWV9NN1NqbHllY0prIiwiY3R5IjoiYXBwbGljYXRpb25cL2pzb24iLCJlbmMiOiJBMjU2R0NNIiwiYWxnIjoiUlNBLU9BRVAtMjU2In0.bX_5aCg25BkWsQ-woCZgxRs0w-6dhyy_ACNJ3XyumlfqhngtDKlYifHR1VWiEWgT4wCBpmruHoTVjxbu9SfHeq6ciZYGXnMxH6Ihn15bnLsSVhtTzOnqWkICuSXnBW1DSOWXxyehn1iiuEWROED4iiLLhHx0teee5w7mSaW94etKjq0PmeOl5W8CF_XpMuvrJG0myEtGJT38KusZi6Omc_gjs72EbBqs9o7i5f0T-mMWScncBy1cuqJQmxRFFot2AZVyxK-UzgV9CfVtytJLhpeAtexVngiDtLaW76_rOaM_jGkUCGcfbXy1RiLSM8XY63bekJhbZJJszDY8wnlYUh6TEimjSP2FjTek9QocnvSkVCg3_t95S5wcoYKU08hdHBVtn_pGRE30EvSeqWOWtVIj9O_QlqrK8nY9i1nvfqxcmj25Lj98mppphETlE2m2__QEoK-1rPE6UjEhs369-u9UGOEuRw7rGfFHGmWUb_17jRJrOjDbdMgvfLmQzsI1elKfpcKlcL6GP0OLglQejfZX1uNUe0_PuVgqI2a5_W0iIsKKMfwLT2Qkq91mRMpjVHzU36YvhZhWmNWchfXojifkBsLrRvz-ACEcJDbtVwoMmnmIUf0FzGR3NZRFdzbTqBtoPRVlX0DZwoiE9YTZsrD44bnE0JYyEu_HZfN7GN4.8N5m-PPDscv_Mopi.oHcKjjnpGMnV6J4lzvn1hW0Ghx8DQKVmzFSGZxYcwtswx7NX3Fg2TWipChtzBeND0GDsHerB4uGpWXh-tXKgKFQf2PG4ETUcyD_dviSCHH2fBuc3NVEIcpwxgG1-aLuYy_lZPPLE8gldFsPp47awJEyIhoFe3kqed_yemQ34kvcJ7q0mpaEj31odrmeWtlM4bxbagJKbrUiGsCTG_67My3P2O1QkTzGnfjFcfEWECCnkLX4wG-51C3Gh8zkM7bjWARgXg21MbPt50u3nBlv02XHLLrJJAsGSUAS0Z9iyuRj75z0ERi1QRwP8OI0hVFqMKevVwXLd5AJrvRmQQJ45wpgCaTtmIikffNGc7JhwWKZIVvy6WBc2JV71guB5V0u_G7glYynQ3d1QnCIAL-PB4eHSpPG20ezX4XVWpff8kHykGJfxQ82D1lT5WKdF99RBb0f0pftuiFrT4gjXqzX21iMV2BKOSExk78umUkU4qx_fC-lK6oFuKoScMQXvS9TWB0OxeayTUu4ttVqDT1ojDSc3T6044-W31VV6LsfmLVd5PaPHgrGpqkfPT1NgG7mw5MeX8OC4.xT8u8qFzZAUVa7Jb9WrDbQ")]
    [TestCase("eyJhbGciOiJSU0EtT0FFUC0yNTYiLCJlbmMiOiJBMjU2R0NNIiwia2lkIjoidnI3Y1dLR2wtZzRXYV9DUkdvd05oQVlXX2dRYi1ha01iaWlneE4wRWtESSIsImN0eSI6ImFwcGxpY2F0aW9uL2pzb24iLCJ6aXAiOiJERUYifQ.R5TmfiDyi7dZhYz9NxduhqZc6GrfAERrDVGNLts6eHhU2SVMTAja1O9Rywrf_thPvbjQslM6ukXB8iI3hI6oWN2EY9SqUjKRto3zL2jdmdcXzfwxJFm1BnObMjYl9Jwy2ogEwLldXQhLAwMxVJWJbf31mZCgnVYC6DbPTEi-GLIkLDEn7Tj5y9iak3TjaE0hIxMk546Dda-q6I9QQ7YlDGV8m0Ijjh21yu2_B9H3Uh9LgIyrTN1Jw2saJRKVJgrtoV6e8WRaj5sDXarLNz6R2LhEtIjYjVioZvjBt1EeXADjcR9m1j3qG8V1f9boVocRTivvvaRvWd2NTH5yFGunDXr40oUmXXWXK1SfKsty85AAjlLOv5ZJsU1vhquA2XDgDVDJ7Nm1qC_9VeW2yCD01Ewh9sGiKscMqP1CbLxyVPfbuViVIe4g0h7Krlb6mZe9L7Sk18cLyFuCre7nYfB5ZToKnSkd8C5-ExMHfrcp9MJ196X6_n7YFKrc-Lzg7tvCRBG-6DFIQ1iqnme6crMN15qwo9VddOJOFXV815vNVyFXLhJAvr85q78aqyaE90qgY2QDNXyZXziTbXLurISL8i_Mdzt0J7cUyrvQLZaK0_pMJJvZOg3LvNalTGdJjUHUFwPxTbe-DRWZuPT81KVNyNa8EKkqox1Ohm_56riMFyU.Od1O-XCSw2Xa8wBN.mHWq8eodcIDXKWjMDDqcNkunVLT0EvjvAWq28gc6aOfKN0zeT0SQczxl0jqcWlziHXE8KmDL7CDYhOZT4rv--lN6wDNj998.YjJfxro1W_ERT7jOvI06yA")]
    public void DecryptDataFromServer_None_ShouldWork(string data) {

        
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        var plain = engine.Decrypt(data);
        Console.WriteLine(plain.plainText);
    }
}
